@@ -17,8 +17,10 @@
 #ifndef _EXTENT_H_
 #define _EXTENT_H_
 
-#include "allocate.h"
-#include "ext4_utils.h"
+#include "ext4_utils/ext4_utils.h"
+
+struct ext4_inode;
+struct block_allocation;
 
 void inode_allocate_extents(struct ext4_inode *inode, u64 len);
 struct block_allocation* inode_allocate_file_extents(
