@@ -109,7 +109,7 @@ int pm_map_workingset(pm_map_t *map, pm_memusage_t *ws_out) {
                                 &flags);
         if (error) goto out;
 
-        if (!(flags & PM_PAGE_REFERENCED))
+        if (!(flags & KPF_REFERENCED))
             continue;
 
         error = pm_kernel_count(map->proc->ker, PM_PAGEMAP_PFN(pagemap[i]),
